@@ -38,7 +38,7 @@ class StrictRedisCluster:
         'hincrby': 'hincrby', 'hincrbyfloat': 'hincrbyfloat', 'hset': 'hset', 'hsetnx': 'hsetnx', 'hmset': 'hmset',
         'incr': 'incr', 'incrby': 'incrby', 'incrbyfloat': 'incrbyfloat', 'linsert': 'linsert', 'lpop': 'lpop',
         'lpush': 'lpush', 'lpushx': 'lpushx', 'lrem': 'lrem', 'lset': 'lset',
-        'ltrim': 'ltrim', 'move': 'move',
+        'ltrim': 'ltrim', 'move': 'move', 'bitop': 'bitop',
         'persist': 'persist', 'publish': 'publish', 'psubscribe': 'psubscribe', 'punsubscribe': 'punsubscribe',
         'rpop': 'rpop', 'rpoplpush': 'rpoplpush', 'rpush': 'rpush',
         'rpushx': 'rpushx', 'sadd': 'sadd', 'sdiff': 'sdiff', 'sdiffstore': 'sdiffstore',
@@ -49,7 +49,7 @@ class StrictRedisCluster:
         'watch': 'watch', 'zadd': 'zadd', 'zincrby': 'zincrby', 'zinterstore': 'zinterstore',
         'zrem': 'zrem', 'zremrangebyrank': 'zremrangebyrank', 'zremrangebyscore': 'zremrangebyscore', 'zunionstore': 'zunionstore',
         'mset': 'mset', 'msetnx': 'msetnx', 'rename': 'rename', 'renamenx': 'renamenx',
-        'del': 'del', 'delete': 'delete', 'ttl': 'ttl', 'flushall': 'flushall', 'flushdb': 'flushdb',
+        'del': 'del', 'delete': 'delete', 'ttl': 'ttl', 'pttl': 'pttl', 'flushall': 'flushall', 'flushdb': 'flushdb',
     }
 
     _dont_hash = {
@@ -83,45 +83,74 @@ class StrictRedisCluster:
 
     def __init__(self, cluster={}, db=0):
         # raise exception when wrong server hash
-        if 'nodes' not in cluster or 'master_of' not in cluster:
+        if 'nodes' not in cluster:
             raise Exception(
                 "rediscluster: Please set a correct array of redis cluster.")
 
         self.cluster = cluster
-        self.no_servers = len(cluster['master_of'])
-        slaves = dictvalues(cluster['master_of'])
+        self.no_servers = len(self.cluster['master_of']) if 'master_of' in self.cluster else len(self.cluster['nodes'])
+
         self.redises = {}
+        redises_cons = {}
+        self.cluster['slaves'] = {}
+
         # connect to all servers
-        for alias, server in iteritems(cluster['nodes']):
-            info = {}
-            try:
-                self.__redis = redis.StrictRedis(db=db, **server)
-                info = self.__redis.info()
-                if alias in slaves and info['role'] == 'master':
-                    raise redis.DataError(
-                        "rediscluster: server %s is not a slave." % (server,))
-            except Exception as e:
-                # if node is slave and is down, replace its connection with its master's
+        for alias, server in iteritems(self.cluster['nodes']):
+
+            if 'master_of' in self.cluster and alias not in self.cluster['master_of']:
+                continue
+
+            server_str = str(server)
+            if server_str in redises_cons:
+                self.redises[alias] = redises_cons[server_str]['master']
+                self.redises[alias +
+                             '_slave'] = redises_cons[server_str]['slave']
+                self.cluster['slaves'][alias +
+                                       '_slave'] = redises_cons[server_str]['slave_node']
+            else:
                 try:
-                    ms = [k for k, v in iteritems(cluster['master_of'])
-                          if v == alias and (('role' in info and info['role'] == 'slave') or cluster['nodes'][k] == cluster['nodes'][v])][0]
-                except IndexError:
-                    ms = None
+                    # connect to master
+                    self.__redis = redis.StrictRedis(db=db, **server)
+                    info = self.__redis.info()
+                    if info['role'] != 'master':
+                        raise redis.DataError(
+                            "rediscluster: server %s is not a master." % (server,))
 
-                if ms is not None:
-                    try:
-                        self.__redis = redis.StrictRedis(
-                            db=db, **cluster['nodes'][ms])
-                        self.__redis.info()
-                    except Exception as e:
-                        raise redis.ConnectionError("rediscluster cannot connect to: %s %s" % (cluster['nodes'][ms], e))
+                    self.redises[alias] = self.__redis
+                    redises_cons[server_str] = {}
+                    redises_cons[server_str]['master'] = self.redises[alias]
 
-                else:
+                    # connect to slave
+                    slave_connected = False
+                    if 'connected_slaves' in info and info['connected_slaves'] > 0:
+                        slave_host, slave_port, slave_online = info[
+                            'slave0'].split(',')
+                        if slave_online == 'online':
+                            try:
+                                redis_slave = redis.StrictRedis(host=slave_host, port=int(slave_port), db=db)
+                                self.redises[alias + '_slave'] = redis_slave
+                                self.cluster['slaves'][alias + '_slave'] = {
+                                    'host': slave_host, 'port': slave_port}
+                                redises_cons[server_str][
+                                    'slave'] = self.redises[alias + '_slave']
+                                redises_cons[server_str]['slave_node'] = self.cluster['slaves'][alias + '_slave']
+                                slave_connected = True
+                            except redis.RedisError as e:
+                                pass
+                                # "RedisCluster cannot connect to: " + slave_host +':'+ slave_port
+
+                    if not slave_connected:
+                        self.redises[alias + '_slave'] = self.redises[alias]
+                        self.cluster['slaves'][alias + '_slave'] = server
+                        redises_cons[server_str][
+                            'slave'] = self.redises[alias + '_slave']
+                        redises_cons[server_str]['slave_node'] = self.cluster[
+                            'slaves'][alias + '_slave']
+
+                except redis.RedisError as e:
                     raise redis.ConnectionError(
                         "rediscluster cannot connect to: %s %s" % (server, e))
 
-            self.redises[alias] = self.__redis
-
     def __getattr__(self, name, *args, **kwargs):
         """
         Magic method to handle all redis commands
@@ -136,7 +165,7 @@ class StrictRedisCluster:
                 # since we don't have "first item" in dict,
                 # this list is needed in order to check hash_tag in mset({"a{a}": "a", "b":"b"})
                 list_ht = []
-                if isinstance(args[0], basestring) or isinstance(args[0], bytes):
+                if isinstance(args[0], (basestring, bytes)):
                     key_type = 'string'
                     list_ht.append(args[0])
                 else:
@@ -185,12 +214,10 @@ class StrictRedisCluster:
 
                 # get the node number
                 node = self._getnodenamefor(hkey)
-                redisent = self.redises[self.cluster['default_node']]
                 if name in StrictRedisCluster._write_keys:
                     redisent = self.redises[node]
                 elif name in StrictRedisCluster._read_keys:
-                    redisent = self.redises[
-                        self.cluster['master_of'][node]]
+                    redisent = self.redises[node + '_slave']
 
                 # Execute the command on the server
                 return getattr(redisent, name)(*args, **kwargs)
@@ -198,7 +225,7 @@ class StrictRedisCluster:
             else:
                 result = {}
                 for alias, redisent in iteritems(self.redises):
-                    if name in StrictRedisCluster._write_keys and alias not in self.cluster['master_of']:
+                    if name in StrictRedisCluster._write_keys and alias.find('_slave') >= 0:
                         res = None
                     else:
                         res = getattr(redisent, name)(*args, **kwargs)
@@ -242,8 +269,7 @@ class StrictRedisCluster:
 
     def object(self, infotype, key):
         "Return the encoding, idletime, or refcount about the key"
-        redisent = self.redises[self.cluster['master_of'][
-            self._getnodenamefor(key)]]
+        redisent = self.redises[self._getnodenamefor(key) + '_slave']
         return getattr(redisent, 'object')(infotype, key)
 
     def _rc_brpoplpush(self, src, dst, timeout=0):
